import resource
from os.path import getsize
from datetime import datetime

from openpyxl import Workbook


def get_maxrss() -> float:
    r = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
    return r // 1024 // 1024  # bytes on MacOS


print(f"{datetime.now().isoformat()} init: {get_maxrss()}MB")
wb = Workbook()
ws = wb.worksheets[0]
txt = "1234567890" * 10

for row in range(1,50000):
    for col in range(1, 20):
        ws.cell(row=row, column=col, value=txt)

print(f"{datetime.now().isoformat()} writed: {get_maxrss()}MB")
wb.save('./output.xlsx')
print(f"{datetime.now().isoformat()} saved: {get_maxrss()}MB")
wb.close()
print(f"{datetime.now().isoformat()} closed: {get_maxrss()}MB")


import unittest
import os
import shutil
import resource
from os.path import getsize
from datetime import datetime
import random

from faker.factory import Factory

import pyexcelize as pe


Faker = Factory.create
fake = Faker()
fake.seed(0)
fake = Faker("ja_JP")


def get_maxrss() -> float:
    r = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
    return r // 1024 // 1024  # bytes on MacOS


class ASeriesOfTest(unittest.TestCase):

    def setUp(self):
        try:
            os.mkdir('__tmp')
        except FileExistsError as e:
            pass

    def test_a_series_of_test(self):
        index = pe.new_file()
        self.assertEqual(1, index)
        pe.save_as(index, '__tmp/test.xlsx')
        self.assertTrue(os.path.exists('__tmp/test.xlsx'))
        pe.close(index)

        index = pe.open_file('./__tmp/test.xlsx')
        self.assertEqual(2, index)
        new_sheet = pe.new_sheet(index, 'Sheet2')
        to_sheet = pe.copy_sheet(index, 1, new_sheet)
        pe.set_active_sheet(index, new_sheet)
        self.assertEqual(to_sheet, pe.get_active_sheet_index(index))
        pe.delete_sheet(index, 'Sheet1')

        pe.set_cell_int(index, "Sheet2", "A1", 1)
        pe.set_cell_int(index, "Sheet2", "A2", 2)
        pe.set_cell_int(index, "Sheet2", "A3", 3)
        pe.set_cell_str(index, "Sheet2", "A4", "hello")

        self.assertEqual('1', pe.get_cell_value(index, "Sheet2", "A1"))
        self.assertEqual('2', pe.get_cell_value(index, "Sheet2", "A2"))
        self.assertEqual('3', pe.get_cell_value(index, "Sheet2", "A3"))
        self.assertEqual('hello', pe.get_cell_value(index, "Sheet2", "A4"))

        style_index = pe.get_cell_style(index, "Sheet2", "A1")
        pe.set_cell_style(index, "Sheet2", "A2", "A2", style_index)
        pe.save(index)


    def tearDown(self):
        shutil.rmtree('__tmp')


class PerformanceTest(unittest.TestCase):

    def setUp(self):
        try:
            os.mkdir('__tmp')
        except FileExistsError as e:
            pass

    def test_performance(self):
        print(f"{datetime.now().isoformat()} init: {get_maxrss()}MB")
        index = pe.new_file()

        txt = "1234567890" * 10

        writer_index = pe.new_stream_writer(index, "Sheet1")
        for row in range(1,50000):
            params = []
            for col in "ABCDEFGHIJKLMNOPQRST":
                params.append(txt)
            pe.set_row(writer_index, f"A{row}", params)

        pe.flush(writer_index)

        print(f"{datetime.now().isoformat()} writed: {get_maxrss()}MB")

        pe.save_as(index, './__tmp/output.xlsx')

        print(f"{datetime.now().isoformat()} saved: {get_maxrss()}MB")

        pe.close(index)

        print(f"{datetime.now().isoformat()} closed: {get_maxrss()}MB")

    def test_template_performance(self):

        print(f"{datetime.now().isoformat()} init: {get_maxrss()}MB")

        index = pe.open_file('./tests/template.xlsx')
        writer_index = pe.new_stream_writer(index, "Sheet1")
        headers = [
            "employee name",
            "company",
            "salary",
        ]
        pe.set_row(writer_index, "A1", headers)
        for row in range(2,500000):
            params = [
                fake.name(),
                random.choice(["Google", "Microsoft", "Apple", "Toyota", "Meta"]),
                random.randint(10000, 10000000),
            ]
            pe.set_row(writer_index, f"A{row}", params)
        pe.add_table(writer_index, "A1", "C499999", dict(
            table_name="テーブル1",
            table_style="TableStyleMedium2",
            show_first_column=True,
            show_last_column=True,
            show_row_stripes=True,
            show_column_stripes=False,
        ))
        pe.flush(writer_index)
        print(f"{datetime.now().isoformat()} writed: {get_maxrss()}MB")
        pe.save_as(index, './__tmp/output.xlsx')
        print(f"{datetime.now().isoformat()} saved: {get_maxrss()}MB")
        pe.close(index)
        print(f"{datetime.now().isoformat()} closed: {get_maxrss()}MB")

    def tearDown(self):
        shutil.rmtree('__tmp')

if __name__ == '__main__':
    unittest.main()


import json
from datetime import datetime, date
from ctypes import CDLL, c_int, c_char_p, create_string_buffer, Structure, c_float, sizeof
from pathlib import Path
import os

from distutils.sysconfig import get_config_var


here = Path(__file__).absolute().parent
ext_suffix = get_config_var("EXT_SUFFIX")
so_file = f'{here}/excelize{ext_suffix}'
lib = CDLL(so_file)
ENCODE = 'utf-8'


__version__ = '0.1.0'


class PyExcelizeError(Exception):
    pass


class ExcelValue(Structure):
    _fields_ = [
        ('int_value', c_int),
        ('float_value', c_float),
        ('str_value', c_char_p),
        ('value_type', c_int),
    ]

ValueType_Int = 0
ValueType_Float = 1
ValueType_String = 2
ValueType_Bool = 3
ValueType_Time = 4
ValueType_Nil = 5


def _param_to_excel_value(value, param):
    if isinstance(value, int):
        param.int_value = value
        param.value_type = ValueType_Int
    elif isinstance(value, str):
        param.str_value = value.encode(ENCODE)
        param.value_type = ValueType_String
    elif isinstance(value, float):
        param.float_value = value
        param.value_type = ValueType_Float
    elif isinstance(value, bool):
        param.int_value = 1 if value else 0
        param.value_type = ValueType_Bool
    elif isinstance(value, (datetime, date)):
        param.str_value = value.isoformat().encode(ENCODE)
        param.value_type = ValueType_Time
    elif value is None:
        param.value_type = ValueType_Nil
    else:
        raise PyExcelizeError('unsupported type')


def new_file() -> int:
    return lib.NewFile()


def open_file(filename: str) -> int:
    return lib.OpenFile(filename.encode(ENCODE))


def save(file_index: int) -> None:
    lib.Save(file_index)


def save_as(file_index: int, filename: str) -> None:
    lib.SaveAs(file_index, filename.encode(ENCODE))


def close(file: int) -> None:
    lib.Close(file)


def new_sheet(file_index: int, sheet_name: str) -> None:
    lib.NewSheet(file_index, sheet_name.encode(ENCODE))


def delete_sheet(file_index: int, sheet_name: str) -> None:
    lib.DeleteSheet(file_index, sheet_name.encode(ENCODE))


def copy_sheet(file_index: int, fromIndex: int, toIndex: int) -> int:
    result = lib.CopySheet(file_index, fromIndex, toIndex)
    if result < 0:
        raise PyExcelizeError('copy sheet error')
    return result


def set_active_sheet(file_index: int, index: int) -> None:
    lib.SetActiveSheet(file_index, index)


def get_active_sheet_index(file_index: int) -> int:
    lib.GetActiveSheetIndex.restype = c_int
    return lib.GetActiveSheetIndex(file_index)


def set_sheet_row(file_index:int, sheet_name: str, axis: str, values):
    ExcelValues = ExcelValue * len(values)
    param = ExcelValues()
    for v, p in zip(values, param):
        _param_to_excel_value(v, p)
    lib.SetSheetRow(file_index, sheet_name.encode(ENCODE), axis.encode(ENCODE), param, len(values))


def set_cell_int(file_index:int, sheet_name: str, axis: str, value: int) -> None:
    lib.SetCellInt(file_index, sheet_name.encode(ENCODE), axis.encode(ENCODE), value)


def set_cell_str(file_index: int, sheet_name: str, axis: str, value: str) -> None:
    lib.SetCellStr(file_index, sheet_name.encode(ENCODE), axis.encode(ENCODE), value.encode(ENCODE))


def set_cell_style(file_index: int, sheet_name: str, h_cell: str, v_cell:str, style: int) -> None:
    lib.SetCellStyle(file_index, sheet_name.encode(ENCODE), h_cell.encode(ENCODE), v_cell.encode(ENCODE), style)


def get_cell_style(file_index: int, sheet_name: str, axis: str) -> int:
    lib.GetCellStyle.restype = c_int
    return lib.GetCellStyle(file_index, sheet_name.encode(ENCODE), axis.encode(ENCODE))


def get_cell_value(file_index: int, sheet_name: str, axis: str) -> str:
    buf_size = 2048
    buf = create_string_buffer(buf_size)
    lib.GetCellValue.restype = c_char_p
    return lib.GetCellValue(file_index, sheet_name.encode(ENCODE), axis.encode(ENCODE), buf, buf_size).decode(ENCODE)


def new_stream_writer(file_index: int, sheet_name: str) -> int:
    return lib.NewStreamWriter(file_index, sheet_name.encode(ENCODE))


def set_row(writer_index: int, axis: str, row: list) -> None:
    ExcelValues = ExcelValue * len(row)
    param = ExcelValues()
    for v, p in zip(row, param):
        _param_to_excel_value(v, p)
    lib.SetRow(writer_index, axis.encode(ENCODE), param, len(row))


def add_table(writer_index: int, h, v, format:dict):
    lib.AddTable(writer_index, h.encode(ENCODE), v.encode(ENCODE), json.dumps(format).encode(ENCODE))


def flush(writer_index: int) -> None:
    lib.Flush(writer_index)


from __future__ import annotations

from glob import glob
from os.path import basename
from os.path import splitext

from setuptools import Extension
from setuptools import setup
from setuptools import find_packages


def _requires_from_file(filename):
    return open(filename).read().splitlines()


setup(
    name="pyexcelize",
    version="0.2.2",
    license="MIT",
    description="A Python library for reading and writing Excel files",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Junichi Yukawa",
    author_email="j.yukawa@gmail.com",
    ext_modules=[Extension('excelize', ['main.go'])],
    build_golang={'root': 'github.com/inunekousapon/pyexcelize', 'strip': False},
    setup_requires=['setuptools-golang'],
    packages=find_packages('./'),
    install_requires=_requires_from_file('requirements.txt'),
    zip_safe=False,
    include_package_data=True,
    py_modules=[splitext(basename(path))[0] for path in glob('./*.py')],
)